# Devices taken from
# https://github.com/mgp25/Instagram-API/blob/master/src/Devices/GoodDevices.php
DEFAULT_DEVICE = 'samsung_galaxy_s7'
DEVICES = {
    # Released on March 2016
    'samsung_galaxy_s7': {
        'instagram_version': '26.0.0.10.86',
        'android_version': 24,
        'android_release': '7.0',
        'dpi': '640dpi',
        'resolution': '1440x2560',
        'manufacturer': 'samsung',
        'device': 'SM-G930F',
        'model': 'herolte',
        'cpu': 'samsungexynos8890',
    },
    # Released on January 2017
    'huawei_mate_9_pro': {
        'instagram_version': '26.0.0.10.86',
        'android_version': 24,
        'android_release': '7.0',
        'dpi': '640dpi',
        'resolution': '1440x2560',
        'manufacturer': 'HUAWEI',
        'device': 'LON-L29',
        'model': 'HWLON',
        'cpu': 'hi3660'
    },
    # Released on November 2016
    'one_plus_3t': {
        'instagram_version': '26.0.0.10.86',
        'android_version': 24,
        'android_release': '7.0',
        'dpi': '380dpi',
        'resolution': '1080x1920',
        'manufacturer': 'OnePlus',
        'device': 'ONEPLUS A3010',
        'model': 'OnePlus3T',
        'cpu': 'qcom'
    },
    # Released on April 2016
    'lg_g5': {
        'instagram_version': '26.0.0.10.86',
        'android_version': 23,
        'android_release': '6.0.1',
        'dpi': '640dpi',
        'resolution': '1440x2392',
        'manufacturer': 'LGE/lge',
        'device': 'RS988',
        'model': 'h1',
        'cpu': 'h1'
    },
    # Released on June 2016
    'zte_axon_7': {
        'instagram_version': '26.0.0.10.86',
        'android_version': 23,
        'android_release': '6.0.1',
        'dpi': '640dpi',
        'resolution': '1440x2560',
        'manufacturer': 'ZTE',
        'device': 'ZTE A2017U',
        'model': 'ailsa_ii',
        'cpu': 'qcom'
    },
    # Released on March 2016
    'samsung_galaxy_s7_edge': {
        'instagram_version': '26.0.0.10.86',
        'android_version': 23,
        'android_release': '6.0.1',
        'dpi': '640dpi',
        'resolution': '1440x2560',
        'manufacturer': 'samsung',
        'device': 'SM-G935',
        'model': 'hero2lte',
        'cpu': 'samsungexynos8890'
    },
}
